#
#   KiCad Support modules for python
#   Note - this will be out-of-date as soon as the new file format comes along,
#   but then KiCad should take on the conversion process for us, and someone
#   can add in new file format support if they so wish :)
#
#

from datetime import datetime

def mm(m):
    """
    A way of generating KiCad units from millimeters
    """
    return int(m * 393.7007874)


class ModuleLibrary:
    def __init__(self):
        self.modules = []
        self.createtime = datetime.now()
    def format(self):
        result  = "PCBNEW-LibModule-V1  {0}\n".format(
            self.createtime.strftime("%d/%m/%Y %H:%M:%S") )
            
        result += "# encoding UTF8\n"
        result += "$INDEX\n"

        for m in self.modules:
            result += "{0}\n".format(m.fields[0].text)
            
        result += "$EndINDEX\n"

        for m in self.modules:
            result += m.format()

        result += "$EndLIBRARY\n"

        return result
        
    def addModule(self, mod):
        self.modules.append(mod)

class Const:
    AUTOPLACE_FIXED = "F"
    AUTOPLACE_MOVEABLE = "~"
    AUTOPLACE_NOT_PLACED = "~"
    AUTOPLACE_PLACED = "P"

    FIELDVIS_VISIBLE = "V"
    FIELDVIS_INVISIBLE = "I"

    LAYER_TOP = 15
    LAYER_INNER1 = 14
    LAYER_INNER2 = 13
    LAYER_INNER3 = 12
    LAYER_INNER4 = 11
    LAYER_INNER5 = 10
    LAYER_INNER6 = 9
    LAYER_INNER7 = 8
    LAYER_INNER8 = 7
    LAYER_INNER9 = 6
    LAYER_INNER10 = 5
    LAYER_INNER11 = 4
    LAYER_INNER12 = 3
    LAYER_INNER13 = 2
    LAYER_INNER14 = 1
    LAYER_BOTTOM = 0
    LAYER_BOTTOM_ADHESIVE = 16
    LAYER_TOP_ADHESIVE = 17
    LAYER_BOTTOM_PASTE = 18
    LAYER_TOP_PASTE = 19
    LAYER_BOTTOM_SILK = 20
    LAYER_TOP_SILK = 21
    LAYER_BOTTOM_MASK = 22
    LAYER_TOP_MASK = 23
    LAYER_DRAWINGS = 24
    LAYER_COMMENTS = 25
    LAYER_ECO1 = 26
    LAYER_ECO2 = 27
    LAYER_PCB_EDGE = 28
    LAYER_29 = 29
    LAYER_30 = 30
    LAYER_31 = 31

    ORIENTATION_0DEG = 0
    ORIENTATION_90DEG = 900
    ORIENTATION_180DEG = 1800
    ORIENTATION_270DEG = 2700

    PADTYPE_STANDARD="STD"
    PADTYPE_SMD="SMD"
    PADTYPE_CONNECTOR="CONN"
    PADTYPE_MECHANICAL="MECH"

    SHAPE_CIRCLE = "C"
    SHAPE_RECTANGLE = "R"
    SHAPE_OBLONG = "O"
    SHAPE_TRAPEZE = "T"

    LAYER_MASK_ALL  = "00C0FFFF"
    LAYER_MASK_TOP  = "00C08000"

    def __init__(self):
        pass

class Pad:
    def __init__(self, position=(0,0), size=(0,0), number=0, orientation=0, shape=Const.SHAPE_CIRCLE, padtype=Const.PADTYPE_STANDARD):
        self.position = position
        self.size = size
        self.number = number
        self.orientation = orientation
        self.shape = shape
        self.padtype = padtype

    def formatSh(self):
        result = "Sh \"{0}\" {1} {2} {3} {4} {5} {6}\n".format(
            str(self.number),
            str(self.shape),
            str(self.size[0]),
            str(self.size[1]),
            str(0),
            str(0),
            str(self.orientation) )

        return result

    def formatDr(self):
        # TODO: Finish drill output
        result = "Dr 0 0 0\n"
        return result

    def formatAt(self):
        result = "At {0} N {1}\n".format( str(self.padtype), str(Const.LAYER_MASK_TOP) )
        return result

    def formatNe(self):
        return "Ne 0 \"\"\n"

    def formatPo(self):
        return "Po {0} {1}\n".format( str(self.position[0]), str(self.position[1]) )

    def format(self):
        result = "$PAD\n"
        result += self.formatSh()
        result += self.formatDr()
        result += self.formatAt()
        result += self.formatNe()
        result += self.formatPo()
        result += "$EndPAD\n"

        return result

class ModuleField:
    def __init__(self):
        self.number = 0
        self.x = 0
        self.y = 0
        self.xsize = 400
        self.ysize = 400
        self.rotation = 0
        self.penwidth = 60
        self.N = "N"
        self.visible = Const.FIELDVIS_VISIBLE
        self.layer = Const.LAYER_TOP_SILK
        self.text = ""

    def format(self):
        """
        Returns the formatted module field record
        """
        
        return "T{0} {1} {2} {3} {4} {5} {6} {7} {8} {9} \"{10}\"\n".format(
            str(self.number),
            str(self.x),
            str(self.y),
            str(self.xsize),
            str(self.ysize),
            str(self.rotation),
            str(self.penwidth),
            str(self.N),
            str(self.visible),
            str(self.layer),
            str(self.text)
        )

class DrawRectangle:
    def __init__(self, centre=(0,0), size=(100,100), lwidth = 100, layer = Const.LAYER_TOP_SILK):
        self.width = size[0]
        self.height = size[1]
        self.centrex = centre[0]
        self.centrey = centre[1]
        self.lwidth = lwidth
        self.layer = layer
        self.segments = []
        self.segments.append(DrawSegment(self.centrex-(self.width/2), self.centrey-(self.height/2), self.centrex+(self.width/2), self.centrey-(self.height/2), self.lwidth, self.layer))
        self.segments.append(DrawSegment(self.centrex+(self.width/2), self.centrey-(self.height/2), self.centrex+(self.width/2), self.centrey+(self.height/2), self.lwidth, self.layer))
        self.segments.append(DrawSegment(self.centrex+(self.width/2), self.centrey+(self.height/2), self.centrex-(self.width/2), self.centrey+(self.height/2), self.lwidth, self.layer))
        self.segments.append(DrawSegment(self.centrex-(self.width/2), self.centrey+(self.height/2), self.centrex-(self.width/2), self.centrey-(self.height/2), self.lwidth, self.layer))

    def format(self):
        """
        Formats a rectangle using KiCad Draw Segments (See DrawSegment Class)
        """
        result = ""

        for d in self.segments:
            result += d.format()

        return result

class DrawSegment:
    def __init__(self, xstart = 0, ystart = 0, xend = 0, yend = 0, width = 100, layer = Const.LAYER_TOP_SILK ):
        self.xstart = xstart
        self.ystart = ystart
        self.xend = xend
        self.yend = yend
        self.width = width
        self.layer = layer

    def format(self):
        """
        Formats a KiCad Draw Segment - formatted as
        DS Xstart Ystart Xend Yend Width Layer
        """
        return "DS {0} {1} {2} {3} {4} {5}\n".format(
            str(self.xstart),
            str(self.ystart),
            str(self.xend),
            str(self.yend),
            str(self.width),
            str(self.layer)
            )




class Module:

    def __init__(self):
        self.x = 0
        self.y = 0
        self.orientation = 0
        self.autoplace1 = Const.AUTOPLACE_MOVEABLE
        self.autoplace2 = Const.AUTOPLACE_NOT_PLACED
        self.libraryname = "DummyLibraryName"
        self.layer = 15
        self.timestamp = 0
        self.fields = []
        self.drawings = []
        self.pads = []

        # Create twelve fields (The first two MUST be used, so allocated text
        # to them, even if it is dummy text)
        i = 0
        while i < 12:
            f = ModuleField()
            f.number = i
            self.fields.append(f)
            i += 1

        for f in self.fields:

            if f.number == 0:
                f.text = "TheModuleName"

            if f.number == 1:
                f.text = "VAL**"

    def addDrawing(self, ds):
        self.drawings.append(ds)

    def addPad(self, p):
        self.pads.append(p)

    def formatPo(self):
        """
        Return the formatted position record for this module
        """

        #    {0}  {1}  {2}                 {3}   {4}       {5}    {6}
        # Po Xpos Ypos Orientation(0.1deg) Layer TimeStamp Attrib1Attrib2
        return "Po {0} {1} {2} {3} {4} {5}{6}\n".format(
            str(self.x),
            str(self.y),
            str(self.orientation),
            str(self.layer),
            str(self.timestamp),
            str(self.autoplace1),
            str(self.autoplace2) )

    def formatLi(self):
        """
        Return the formatted library name record
        """
        return "Li {0}\n".format(self.libraryname)

    def formatSc(self):
        """
        Return the formatted autoplace timestamp record
        """
        return "Sc 00000000\n"
        
    def formatOp(self):
        """
        Return the Autoplace rotation permission record
        """
        return "Op 0 0 0\n"

    def formatT(self):
        """
        Return the formatted text field records
        """
        result = ""
        
        for f in self.fields:
            if f.text != "":
                result += f.format()
            
        return result

    def formatDrawings(self):
        """
        Return the formatted draw segment records
        """

        result = ""

        for d in self.drawings:
            result += d.format()

        return result

    def formatPads(self):
        """
        Return the formatted pad records
        """

        result = ""

        for p in self.pads:
            result += p.format()

        return result

    def format(self):
        """
        Returns a string that will be a KiCad module formatted item
        """
        result  = "$MODULE {0}\n".format(self.fields[0].text)
        result += self.formatPo()
        result += self.formatLi()
        result += self.formatSc()
        result += self.formatOp()
        result += self.formatT()
        result += self.formatDrawings()
        result += self.formatPads()
        result += "$EndMODULE {0}\n".format(self.fields[0].text)
        
        return result
