#
#   Python script to create DPAK footprints
#
#   (c)2012 Brian Sidebotham - www.valvers.com
#
#   The information for the DPAK footprints was taken from PDFs linked from:
#   http://www.irf.com/package/pkhexfet.html
#

import kicad as KiCad
from kicad import mm as mm

l = KiCad.ModuleLibrary()

# DPAK
# from http://www.diodes.com/zetex/_pdfs/3.0/pack/DPAK.pdf
dpak3 = KiCad.Module()

pincount = 3
linewidth = 0.2 # in mm

# Set the module name
dpak3.fields[0].text = "DPAK{0}".format(str(pincount))
dpak3.fields[0].y = -mm(2.5)
dpak3.fields[0].visible = KiCad.Const.FIELDVIS_INVISIBLE

# Move the reference default location away from the pads
dpak3.fields[1].y = -mm(12)

# Mark the body outline which is the
dpak3.addDrawing(
    KiCad.DrawRectangle(
        size = (mm(6.2 + 0.5), mm(12.4 + 0.5)),
        centre = (mm(0), mm(-4.4)),
        lwidth = mm(linewidth),
        layer = KiCad.Const.LAYER_TOP_SILK ) )

dpak3p1 = KiCad.Pad( position = (mm(-2.285), mm(0)),
                size = (mm(1.6), mm(3.0)),
                number = 1,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

#dpak3p2 = KiCad.Pad( position = (mm(-2.285), mm(0)),
#                size = (mm(1.6), mm(3.0)),
#                number = pin_number,
#                orientation = KiCad.Const.ORIENTATION_0DEG,
#                shape = KiCad.Const.SHAPE_RECTANGLE,
#                padtype = KiCad.Const.PADTYPE_SMD )

dpak3p3 = KiCad.Pad( position = (mm(2.285), mm(0)),
                size = (mm(1.6), mm(3.0)),
                number = 3,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

dpak3p4 = KiCad.Pad( position = (mm(0), mm(-7.18)),
                size = (mm(5.8), mm(6.2)),
                number = 4,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

dpak3.addPad(dpak3p1)
#dpak3.addPad(dpak3p2)
dpak3.addPad(dpak3p3)
dpak3.addPad(dpak3p4)

# D2PAK
d2pak3 = KiCad.Module()

pincount = 3
linewidth = 0.2 # in mm

# Set the module name
d2pak3.fields[0].text = "D2PAK{0}".format(str(pincount))
d2pak3.fields[0].y = -mm(2.5)
d2pak3.fields[0].visible = KiCad.Const.FIELDVIS_INVISIBLE

# Move the reference default location away from the pads
d2pak3.fields[1].y = -mm(17.5)

# Mark the body outline which is the
d2pak3.addDrawing(
    KiCad.DrawRectangle(
        size = (mm(11.8 + 0.5), mm(18.2 + 0.5)),
        centre = (mm(0), mm(-6.9)),
        lwidth = mm(linewidth),
        layer = KiCad.Const.LAYER_TOP_SILK ) )

d2pak3p1 = KiCad.Pad( position = (mm(-2.54), mm(0)),
                size = (mm(2.08), mm(3.81)),
                number = 1,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

#d2pak3p2 = KiCad.Pad( position = (mm(-2.285), mm(0)),
#                size = (mm(1.6), mm(3.0)),
#                number = pin_number,
#                orientation = KiCad.Const.ORIENTATION_0DEG,
#                shape = KiCad.Const.SHAPE_RECTANGLE,
#                padtype = KiCad.Const.PADTYPE_SMD )

d2pak3p3 = KiCad.Pad( position = (mm(2.54), mm(0)),
                size = (mm(2.08), mm(3.81)),
                number = 3,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

d2pak3p4 = KiCad.Pad( position = (mm(0), mm(-11.43)),
                size = (mm(11.5), mm(9)),
                number = 4,
                orientation = KiCad.Const.ORIENTATION_0DEG,
                shape = KiCad.Const.SHAPE_RECTANGLE,
                padtype = KiCad.Const.PADTYPE_SMD )

d2pak3.addPad(d2pak3p1)
#d2pak3.addPad(d2pak3p2)
d2pak3.addPad(d2pak3p3)
d2pak3.addPad(d2pak3p4)

# D3PAK

l.addModule( dpak3 )
l.addModule( d2pak3 )

print l.format()

